﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

DESCRIPTION:
    
This is a script that shows basic techniques
for working with document objects. It does not
manipulate any documents... it just looks to the
session to see if there is an active document
and does a little reporting about what it finds.

*/

//Here is how to get the active document; that is, the
//document that has the current focus. "app" is the
//the parent "session" object that exists automatically
//when FrameMaker is running. "ActiveDoc" is a property
//of the session and will return a document object if an active
//document exists. All of this stuff is built in and described (briefly)
//in the scripting guide.

//If the scripting guide completely baffles you
//right now, it might be good to take a second look after you
//run and study this script. Search for the properties shown in this script.
//You might get a glimmer of enlightenment.

var doc = app.ActiveDoc;

//At this point, we either have an active document (if there was one)
//or an invalid object if not. You always have to test and account for
//this sort of thing. If you attempt further actions on an invalid object,
//your script will probably crash. We can test for a valid object with the
//build-in "ObjectValid()" method (look for it in the scripting guide).

if(doc.ObjectValid() == true) 
{
    //At this point, we know we have a valid active document. Let's just report
    //the filepath, available as the "Name" property of a document object. 
    //Again, look for that in the scripting guide. Note that the"\n\n" business
    //is just how you put line breaks in a prompt message. In programming
    //terms, "\n" is called an "escape sequence" that inserts a newline character
    //which prompts and various other string rendering entities will respect.
    var promptMessage = "The currently-active document is:\n\n" +
        doc.Name;
    
    alert(promptMessage);
}

//Otherwise, if there is no active document, report that.
else
{
    alert("Sorry, no active document could be found.");
}

//That's the end of the script. Note that the built-in session
//object ("app") also has an "ActiveBook" property. That should
//start a couple wheels spinning....

alert("Script complete!");