﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

DESCRIPTION:
    
This is a script that demonstrates how to browse
for a document and open it.

Like all scripts, if a particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Produce the built-in file browser prompt.
//This is provided by the FM interface and all
//we need to do is ask for it. Note that you
//can do fancy things with it like set the starting
//folder, which normally you would want to do
//for usability. For simplicity here, we'll let FM select
//some default starting folder for us.
//
//There are several variations possible with this browser.
//This one is specifically designed to allow you to pick a file
//from the file system and then click an Open button. The
//parameter "Constants.FV_ChooseOpen" determines this.
//Search the scripting guide for this parameter to learn more
//about what this browser can do and look like.
//
//This function will return the selected path as a string if
//the user goes all the way through, or an empty string
//if the user clicks Cancel, etc.

var path = ChooseFile("Browse for file", "", "", Constants.FV_ChooseOpen);

//If we have a path...
if(path != "")
{
    //Let's try to open the file. If successful, this function
    //will return the document object.
    //Note that "SimpleOpen() is the simplest way to open a file
    //but also the most restrictive. If the slightest thing is amiss,
    //such as the file is a different FM version, has some kind of
    //error, etc., it will fail. There is another function, "Open()" which
    //is fully configurable to handle all these situations. But because it
    //is much more complicated to use, we'll stick with "SimpleOpenI()"
    //for this sample.
    //There is another script later in the batch that shows an example of
    //Open().
    var doc = SimpleOpen(path, "");
    
    //If we successfully opened the file, let's report it.
    if(doc.ObjectValid() == true)
    {
        alert("Successfully opened the file:\n\n" + doc.Name);
    }

    //Otherwise, report the failure.
    else
    {
        alert("Couldn't open the file! Check to see if it is an FM document " +
            "and if somebody is already using it. The selected path was:\n\n" + path);
    }

    alert("Script complete!");
}

//Here's a little prompt if the user cancels the browser prompt
else alert("Action aborted.");


