﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:

This is a script that deletes a paragraph format 
from the active document.

As written, it will delete the "Body" 
format from the catalog. However, you
will note that nothing visibly changes, even 
for paragraphs that are assigned the Body format.
Why? Because when you assign a format to 
a paragraph, you are basically just applying the
properties of the format to the text. Afterwards,
there is no inherent link between the format
and the paragraph. That's why you can 
always make format overrides at will. So,
the formatting originally applied with the format
remains, but you will notice that "Body" no longer 
appears in the format catalog. Furthermore,
if you reimport the EDD (structured docs only),
the original Body paragraphs will lose their formatting.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//For convenience, let's set the 
//format name as a variable. That way
//we can easily make the the script work on 
//a different format by just changing it once here.
var formatName = "Body";

//Get the active document
var doc = app.ActiveDoc;

//If we have an active document, let's
//attempt to find the format and delete it.
if(doc.ObjectValid() == true)
{
    //This is a special ExtendScript method for FrameMaker,
    //used to find something that has a "name", like a format.
    var pgfFormat = doc.GetNamedObject(Constants.FO_PgfFmt, formatName);
    
    //If we found it, we can use a simple Delete() method
    //on the format object to delete it. Many types of objects have this
    //method.
    if(pgfFormat.ObjectValid() == true)
    {
        pgfFormat.Delete();
        
        alert("'" + formatName + "' paragraph format successfully deleted.");
    }

    //Otherwise, if we did not find the format, let's report that.
    else
    {
        alert("Could not find the '" + formatName + 
            "' format in the active document.");
    }
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");

