﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: (any)

DESCRIPTION:
    
This is a script that demonstrates how to iterate
over all paragraph formats in a document.
The script will report all the format names it
finds, up to 25.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document
var doc = app.ActiveDoc;

//If we have an active document, let's
//attempt to find the format and delete it.
if(doc.ObjectValid() == true)
{
    //Initialize these counters. We'll only report up to 25 formats.
    var totalFormats = 0;
    
    //Let's initialize our prompt string. We'll add to it as
    //we iterate over the paragraph formats.
    var msg = "Paragraph formats found: \n\n";
    
    //Let's ask the document for the first paragraph format
    //in its internal list. This list is in no particular order.
    var pgfFormat = doc.FirstPgfFmtInDoc;
    
    //If we found it, we can use a simple Delete() method
    //on the format object to delete it. Many types of objects have this
    //method.
    while(pgfFormat.ObjectValid() == true)
    {
        //Increment our counter
        totalFormats = totalFormats + 1;
        
        //Add the name to the prompt message, if we haven't
        //exceeded our limit
        if(totalFormats < 26) 
            msg = msg + " - " + pgfFormat.Name + "\n";
            
        //...and get the next format in the document.
        pgfFormat = pgfFormat.NextPgfFmtInDoc;
    }

    //Now, let's report. If we did not report all the names
    //because the limit was exceeded, let's add that to 
    //the prompt message.
    if(totalFormats > 25)
        msg = msg + "...and " + (totalFormats - 25) 
            + " more format(s) not reported\n";
            
    //And the final part of the message, announcing the
    //total number of formats found
    msg = msg + "\n" + totalFormats + " format(s) found.";
    
    //Finally, report
    alert(msg);
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");


