﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:
    
This is a script that applies the BoldRed
character format to the 10th through
22nd characters of the first paragraph
of the active document. It is intended to demonstrate
how to apply a format to a certain area of text,
a task that is deceptively more complex than
doing it with the GUI.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
var doc = app.ActiveDoc;

//If we have an active document, let's
//do the requested task.
if(doc.ObjectValid() == true)
{
    //Applying a character format in a script means 
    //retrieving the properties from the format
    //and applying them to a range of text.
    //We can first look up the character format by name.
    charFmt = doc.GetNamedCharFmt("BoldRed");
    
    //If we found the format, we can proceed.
    if(charFmt.ObjectValid() == true)
    {
        //Get the character format properties
        var props = charFmt.GetProps();
        
        //Now, let's get the first paragraph and establish
        //where we want to apply these properties.
        //We'll need to set up a text range for this, which
        //is really just a starting point and an ending point 
        //in the text.
        var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;

        //This bit of complexity is nearly equivalent
        //to selecting the 10th through 22nd character
        //in the paragraph represented by 'pgf', which
        //we have set as the first paragraph in the main
        //flow. Text ranges are complicated and can
        //take some time to understand.
        var textRange = new TextRange();
        textRange.beg.obj = pgf;
        textRange.beg.offset = 9;
        textRange.end.obj = pgf;
        textRange.end.offset = 22;
        
        //The document object provides a special method
        //just for the application of text properties. Naturally,
        //the range of text where to apply these properties
        //is an important input.
        doc.SetTextProps (textRange, props);

        alert("Format applied.");
    }
    
    //If we didn't find the character format, a warning....
    else alert("Could not find the Bold character format!");
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");


