﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm

DESCRIPTION:

This is a script that will:

 - Insert some text at the beginning of the first paragraph
 - Insert some text in the middle of the second paragraph
 - Insert some text at the end of the third paragraph
 - Replace the text in the fourth paragraph
 
The script might work on other files too but
IT DEFINITELY WILL NOT WORK CORRECTLY
ON THE STRUCTURED SAMPLE. 
It might even crash FrameMaker.

Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
var doc = app.ActiveDoc;

//If we have an active document, let's
//do the requested task.
if(doc.ObjectValid() == true)
{
    //We'll start by getting the first paragraph in the main flow,
    //where we'll do the first operation.
    var pgf = doc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;
    
    //The operation for the first paragraph 
    //will be an insertion at the beginning.
    
    //When adding text to a document, a primary input is
    //where to add the text. Basically, you need to 
    //do a programmatic equivalent of  setting the insertion
    //point somewhere. The location of an insertion point
    //can always be identified by the parent paragraph and the
    //character offset from the beginning of that paragraph,
    //so within a script we must explicitly configure this.
    
    //We already have the parent paragraph object. The
    //offset from the beginning will be zero, since we want
    //to insert at the beginning. In the scripting world,
    //we create a "text location" structure to identify
    //this location and then feed it to the method to add text.
    
    //Text locations and text ranges can be tricky to understand.
    //If this takes some time for you to get, don't be discouraged.
    //It is normal to be confused by this at first, even for someone
    //who is an experienced software developer.
    var textLoc = new TextLoc();
    textLoc.obj = pgf;
    textLoc.offset = 0;
    
    //So now we have the programmatic equivalent of an
    //insertion point. Let's insert the text at that point. The document
    //object has an AddText() method that simply needs to know
    //what text to add and where to add it.
    doc.AddText(textLoc, "--NEW TEXT AT THE BEGINNING!-- ");
    
    //Get the next paragraph, where we will insert text in the middle.
    pgf = pgf.NextPgfInFlow;
    
    //The operation here is similar, except that we will add text in
    //the "middle." Let's consider 10 characters in to be the middle.
    textLoc.obj = pgf;
    textLoc.offset = 10;
    doc.AddText(textLoc, " --NEW TEXT IN THE MIDDLE!-- ");
       
    //Get the next paragraph, where we will insert text at the end.
    pgf = pgf.NextPgfInFlow;
    
    //The operation here is similar, except now there is a catch.
    //Right now, we don't know exactly how many characters
    //are in this paragraph, so how can we set the offset at the
    //end? Well, we'll use a little trick. We will first
    //select the paragraph using a text range and 
    //a special constant that allows use to indicate the
    //whole paragraph. Then, we'll query the text range
    //back from the document which will retrieve the
    //actual offset, which we can use to add the text.
    //If this doesn't make any sense initially, consider
    //yourself normal. Some things are a bit of a game
    //to get working right.
    
    //One thing to keep in mind that might help...
    //a text range is nothing but a pair of text 
    //locations, stored together in a single data structure.
    
    var textRange = new TextRange;
    textRange.beg.obj = textRange.end.obj = pgf;
    textRange.beg.offset = 0;
    textRange.end.offset = Constants.FV_OBJ_END_OFFSET;
    
    //Let's set the text selection, equivalent to 
    //selecting the whole paragraph with a mouse.
    //Then get the text range again from it.
    doc.TextSelection = textRange;
    textRange = doc.TextSelection;
    
    //And now finally add the text at the end, using the
    //the second text location in the text range structure,
    //that is, the end of the paragraph. We actually have to
    //decrement the offset by one, to get us to the left of the 
    //paragraph mark.
    textRange.end.offset--;
    doc.AddText(textRange.end, " --NEW TEXT AT THE END!--");
       
    //Get the next paragraph, where we will replace text.
    pgf = pgf.NextPgfInFlow;
    
    //To replace text, it is a two-part operation... first
    //delete the text you want to replace, then insert
    //the new text where the old text used to be.
    //To delete text, we first have to "select" it, then
    //we can call a Clear() method on the document
    //to delete that text. It is very similar to selecting
    //the text in the GUI and pressing Delete.
    //Again, we'll use a text range structure for this.
    //In this case, we want to replace the whole
    //paragraph, so we will select the whole thing
    //for deletion.
    
    //There is some trickery here that is not
    //fully explained in these comments. It is too
    //involved to describe at this point... so just
    //expect that it will take more time for you to 
    //to make full sense of it.
    textRange.beg.obj = textRange.end.obj = pgf;
    textRange.beg.offset = 0;
    textRange.end.offset = Constants.FV_OBJ_END_OFFSET;
    doc.TextSelection = textRange;
    textRange = doc.TextSelection;
    textRange.end.offset--;
    doc.TextSelection = textRange;

    //Now we have all the text selected minus the paragraph mark,
    //and can clear it
    doc.Clear(0);
    
    //And now finally add the new text. We have nothing
    //left but the paragraph mark, so we want to add it to 
    //the left of that. This would naturally be at the offset
    //of zero, which we already have as the beginning
    //text location in the text range.
    doc.AddText(textRange.beg, "--NEW REPLACEMENT TEXT!--");
       
}

//Else, if we did not find an active document, let's
//warn.
else
{
    alert("No active document found. Cannot continue.");
}

alert("Script complete!");

