﻿/*  ....Commentary....

REQUIRED ACTIVE FILE: 

  SampleFile_UNSTRUCTURED.fm
     ....with the graphic on the first page selected!
     Select the graphic, not the anchored frame!

DESCRIPTION:
    
This is a script that resizes and rotates the 
selected graphic in regular intervals.
It is intended to show simple graphic operations.
You should select the graphic in the sample
document before running the script.


Like all scripts, if some particular aspect is
described in detail in some "preceeding"
script in the batch, it will be described only
briefly here.

*/

//Get the active document.
//and the currently-selected graphic
var graphic = null;
var doc = app.ActiveDoc;
if(doc.ObjectValid() == true)
{
    graphic = doc.FirstSelectedGraphicInDoc;
    
    //If we did get a selected graphic, let's make
    //sure it is a referenced graphic. That previous
    //property could return a selected anchored frame,
    //a selected text frame, a selected drawing object,
    //or some other things. This sample is designed
    //to work on the referenced graphic in the sample
    //file, so let's test. Here's how to do it. As an aside,
    //this is somewhat of a kludge on the part of FM
    //ExtendScript. There should be a method to 
    //test the object directly.
    if(graphic.constructor.name != "Inset")
        graphic = null;
}    


//If we have everything we were looking for, let's do the deed.
if(graphic != null && graphic.ObjectValid() == true)
{
    //Alert about what we are getting ready to do. We'll do the resize first.
    alert("Resizing the graphic in 1 cm intervals, from 1 cm to 7 cm.");
        
    //Conveniently for this sample, 
    //the graphic is a perfect square. Let's size it
    //down to 1 cm by 1 cm, then size up.
    //
    //Graphic size is specified by this FM unit known
    //as point "metrics". It's a little complicated to understand. For the sake
    //of this sample, just know that a metric value of 1857713 equals 1 cm,
    //and the conversion is linear from there.
    graphic.Width =  1857713; 
    graphic.Height =  1857713;  
   
    //Now run this loop to iterate from 1 to 7 (cms),
    //multiplying the current value by 1857713 to
    //get the actual value to set in the graphic properties.
    for(var i = 1; i < 8; i++)
    {
        graphic.Width = i * 1857713;
        graphic.Height = i * 1857713;
        
        //Let's pause for a half second so
        //we can watch this happen. ExtendScript
        //has this little sleep method build in to pause
        //processing. The argument is the number of
        //milliseconds to sleep.
        $.sleep(500);
    }



    //Now let's do rotation
    alert("Rotating the graphic in 30 degree intervals, all the way around.");

    //Make sure the graphic is not rotated at all to start.
    graphic.Angle = 0;

    //There are 12 increments of 30 degrees in 360 degrees.
    //So let's run a loop for 12 iterations, increasing the rotation
    //by 30 degrees each time.
    for(var i = 0; i < 12; i++)
    {
        //Now some more funny business with metrics...
        //decimal 65536 equals 1 degree in metric land.
        graphic.Angle = i * 30 * 65536 ;
        
        //Let's pause for a half second so
        //we can watch this happen. ExtendScript
        //has this little sleep method build in to pause
        //processing. The argument is the number of
        //milliseconds to sleep.
        $.sleep(500);
    }

     //At this point, the graphic should be at 330 degrees.
     //Let's bring it back to a full 360 degree rotation
     //(0 degree rotation).
    graphic.Angle = 0;

}

//Else, if we did not find an active document, land 
//never attempted the socket connection, let's warn.
else
{
    alert("No active document or no referenced graphic selected. " +
        "Cannot continue.");
}

alert("Script complete!");
