﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:

This script creates very basic palette (modeless dialog box)
with two buttons, using a technique called "resource specification."
It is an alternative to coordinate-based layouts and offers an
easier path to resizable dialog boxes, where the controls
within also resize and reposition as applicable.
The point is to demonstrate the very basics of creating
a user interface, which can get enormously complicated
depending on what you want to build.

This palette is resizable and the buttons will resize with it.

See the JavaScript Tools Guide under the Help
menu of the ExtendScript Toolkit window for more information.
The resource specification concept is really nice but
can be tough to understand.

*/

LaunchPalette();

function LaunchPalette()
{
    //Define the dialog box as a "resource specification".
    //This big long string is effectively a roadmap on how
    //to build the dialog box, with details on where controls
    //should go and what should be their behavior when
    //the dialog box is resized. The concepts are generally
    //simple once you understand them, but the process can
    //get very complex with bigger layouts. See the JavaScript
    //Tools Guide for more information.
	var resource =
		"""palette {
			
              properties:{ closeButton:true, maximizeButton:false,
				minimizeButton:false, resizeable:true },
			
              orientation:'row', spacing:15, margins:10,
              
              helloButton: Button { 
                  text:'Say Hello', alignment:['fill','fill'] },
             
              closeButton: Button { 
                  text:'Close', alignment:['fill','fill'] },
        }""";   
    
    //Define the palette resource based on the specification/
    var palette = new Window (resource, "My Palette");
    
    //Do some things here to establish the layout and
    //display the palette. The final "resize()" call is what
    //implements all the resizing rules once the palette is
    //displayed at the specified location and dimensions.
    palette.layout.layout(true);
    palette.show();
    palette.bounds = {x:100, y:100, width:158, height:38};
    palette.layout.resize();

    //Now establish the "callbacks", that is, functions that execute when
    //controls are operated. For example, this first one defines what to do
    //when somebody clicks the "Say Hello". There are a wide
    //variety of user actions that you can respond to... in this case, we only
    //care about a click. Because we have not specified any code for any of
    //the other types of actions (callbacks), nothing happens when those
    //actions are performed.
    palette.helloButton.onClick = function() 
    {
        alert("Hi there! You are using FrameMaker version "
            + app.VersionMajor + "." + app.VersionMinor + "." + app.VersionRevision
            + ".\n\nYour username/login is \"" + app.UserLogin + "\"."
            + "\n\nThe product is installed at the following location:\n\n  "
            + app.HomeDir
            + "\n\nHave a nice day!");
	};

    //Callback for clicking the Close button, just want to close the palette.
    palette.closeButton.onClick = function() 
    {
        palette.close();
        palette = null;
	};

    //This gets called by close() above and when
    //you click X in the upper right corner.
    palette.onClose = function() 
    {
        palette = null;
	};

    //The application of resizing rules does
    //not happen automatically. You need to 
    //respond to the "onResize" event which is
    //called when the user resizes the palette.
    //In this case, we just want resizing rules reapplied,
    //which is done with the resize() method of the
    //child layout object. In other cases, you might
    //want some more complicated things to happen and
    //could specify them here.
    palette.onResize = function() 
    {
        palette.layout.resize();
	}
 }



