﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (any)

DESCRIPTION:

This is a script that allows you to define commands/shortcuts
for the:

- Application of a character formats and/or
- Wrapping of structural elements

When you execute a command defined by this script,
it acts upon the currently-selected text. It is not quite
as fancy as the native FM pods in that it won't work
on a whole table cell, row, or stuff like that. You have to
have text selected.

The script was originally created to "hijack"
common shortcuts such as Ctrl+B to apply
a bold character format, rather than to do a simple format
override. In many environments, format overrides
are (logically) prohibited, so why give up Ctrl+B
when you can make it apply a character format instead?
Beyond Ctrl+B, the script is customizable that you can use
assign any shortcut to a formatting action, not
just the common ones.

This type of little amenity can make a big difference
over time to an author. It is a perfect example of how
a little bit of customization can make a big difference
in the workflow.

From the "factory," the script is configured with a
two shortcuts:

 - Apply a "Bold" character format with Ctrl+B
 - Wrap an <i> element with Ctrl+I
 
 In your environment, you will certainly need to 
configure it to meet your specific needs. 
All configuration is done in the SetGlobals()
function, just a bit further below. Instructions
are contained therein. Please be sure to read
all of them carefully so the script does what you
want and does not do any damage. If you have
trouble figuring it out, please contact West Street
for help.


*/



//This is here because I kept having problems with
//ES recognizing the Command() callback in the script.
//This seems to help. I think there is a bug somewhere.
function Command(cmd) { ExecuteCommand(cmd) };

//Set the globals array, so that things
//like the element tag, character format 
//settings, etc. will remain available
//to all functions in the script. This is really
//just a collection of global settings
var sGlobals = new SetGlobals();

//Define all the commands to execute some
//formatting action.
DefineCommands();

alert("The formatting commands/shortcuts "
    + "script has run. Any commands configured in "
    + "the script have been created.\n\nIn the future, "
    + "if you want "
    + "the script to run without this message, go into "
    + "the code and comment it out.");

//This is the function used to create all
//the global settings, called when the script
//executes. It is set up to allow the creation 
//of a "psuedo-object" that remains available to
//all functions when commands are executed
//in the future.

//  ***** Hey you, L@@K!! ******
//This is where you should configure all the
//commands that you want the script to establish.
//Follow the comment instructions within this function.
//PLEASE BE SURE TO READ ALL OF THE
//COMMENTS THROUGHOUT THIS FUNCTION!
function SetGlobals()
{
    //Variable declarations. Don't touch these.
    var descriptions = new Array();
    var shortcuts = new Array();
    var elemTags = new Array();
    var attrNames = new Array();
    var attrVals = new Array();
    var chrFormats = new Array();
    
    /*
    Below here is where all customization 
    should occur. Effectively, there should be
    a single "descriptions - chrFormats" block for each
    active shortcut.  Set  up a block for each command
    you want to define. You should edit the content  within the
    double quotes, where the parameters are as follows:
    
      - descriptions - A description to appear in the 
                                    optional shortcuts palette. This can be any text
                                    at all. More info later.
      - shortcuts - The keyboard sequence for the shortcut. More
                              information on how shortcuts are constructed
                              appears later on. The shortcut is not required
                              if you only want the command to appear in 
                              the shortcuts palette. If you do not want to 
                              assign a keyboard shortcut, leave this as
                              an empty string.
      - elemTags - The element tag to wrap over the selected
                                 text when the shortcut is invoked. If you plan
                                 to apply a character format instead, leave this
                                 as an empty string.
       - attrNames - For an element wrap, an optional attribute to set.
                                  If you do not want to set and attribute or you are
                                  applying a character format instead, leave this as
                                  an empty string.
      - attrVals - If you specified an attribute name above, the value to set.
                           Otherwise, leave this as an empty string.
      - chrFormats - The character format to apply. If you are using an 
                                   an element tag instead, this should be an empty
                                   string. Do not specify both an element tag and a
                                   character format.
                                   
     A shortcut is represented as a string that begins with at 
     least one special character, then a "normal" character.
     Here are the special characters that work well on Windows:
     
     \\!     - Esc key
     ^      - Ctrl key
     +      - Shift key
     
     Here are some examples:
     
     ^b                - Ctrl + B (press and hold keys in sequence)
     ^+b              - Ctrl + Shift + B  (press and hold keys in sequence)
     \\!b               - Esc + B (Esc first, then B)
     \\!bb            - Esc + B + B (Esc first, then B, then B again)
     
     IMPORTANT! Any shortcut you specify here will override 
     any native, default shortcut with the same sequence! For 
     example, if you define Ctrl + B, then the normal functionality
     of Ctrl + B will be overridden and only the code in this
     script will be executed.
     
     Here is a complete example of setting up Ctrl + B command
     to apply a Bold character format to the selected text:
    
    descriptions.push("Bold (Apply Bold format)"); 
    shortcuts.push("^b"); 
    elemTags.push("");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("Bold");
    
     Here is a complete example of setting up Ctrl + B command
     to wrap the selected text in a <b> element:
    
    descriptions.push("Bold (Wrap <b> element)"); 
    shortcuts.push("^b"); 
    elemTags.push("b");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("");
    
     Here is a complete example of setting up Ctrl + B command
     to wrap the selected text in a <ph> element and 
     set a 'props' attribute to 'strong':
    
    descriptions.push("Bold (Wrap <ph props='strong'> element)"); 
    shortcuts.push("^b"); 
    elemTags.push("ph");
    attrNames.push("props");
    attrVals.push("strong");
    chrFormats.push("");
    
     Here is a complete example of setting up an Esc + E + E command
     to apply an Emphasis character format,
    
    descriptions.push("Emphasis (Apply Emphasis format)"); 
    shortcuts.push("\\!ee"); 
    elemTags.push("");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("Emphasis");
     
     Here is a complete example of setting up a command
     to apply an Emphasis character format, but with no
     shortcut. This means that the command will only appear
     in the shortcuts palette.
    
    descriptions.push("Emphasis (Apply Emphasis format)"); 
    shortcuts.push(""); 
    elemTags.push("");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("Emphasis");
     
      */
     
    //Here are the the two factory-default shortcuts...
    
    //Apply a Bold character format with Ctrl+B
    descriptions.push("Bold (Ctrl+B, apply char fmt)"); 
    shortcuts.push("^b"); 
    elemTags.push("");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("Bold");
    
    //Wrap an <i> element with Ctrl+I
    descriptions.push("Italics (Ctrl+I, apply <i> element)"); 
    shortcuts.push("^i"); 
    elemTags.push("i");
    attrNames.push("");
    attrVals.push("");
    chrFormats.push("");
    
    //Here is where you can define optional undo
    //and redo shortcuts, which apply to actions
    //performed by this script only. You get a single
    //undo only, but unlimited redos. Actually, with respect
    //to the behavior of this script, "redo" really means
    //"execute the most recent command again, but execute
    //on whatever text is currently selected."
    //If you do not
    //want to define these, specify empty strings.
    //You can define none, one, or both.
    //Even if you do not specify shortcuts, these commands
    //will appear in the shortcuts palette and be functional.
    var undoShortcut = "^+z";
    var redoShortcut = "^+r";
    
    //Here is an optional shortcut  to unwrap the currently-
    //selected element. It is just a little extra feature that
    //has nothing to do with any other action performed
    //by the script. It's in here because FrameMaker's
    //native commands to unwrap are clunky to use.
    var unwrapShortcut = "";
    
    //Here is the shortcut to launch the shortcuts palette,
    //from which you can execute any command defined
    //above, whether or not a keyboard shortcut is defined.
    //It may be handy for executing commands without
    //shortcuts, that you want to use redo command for
    //afterwards
    var catalogShortcut = "\\!fc";
    
    //That's it for the customization area. Do not change anything 
    //below here! Everything else in the script should be left alone,
    //unless you are prepared to break something :)
    var msgHeader = ":: Formatter script message ::\n\n";
    this.msgHeader = msgHeader;
    
    
    this.descriptions = descriptions;
    this.shortcuts = shortcuts;
    this.elemTags = elemTags;
    this.attrNames = attrNames;
    this.attrVals = attrVals;
    this.chrFormats = chrFormats;
    
    this.undoShortcut = undoShortcut;
    this.redoShortcut = redoShortcut;
    this.unwrapShortcut = unwrapShortcut;
    this.catalogShortcut = catalogShortcut;
    
    var redoIndex = -1;
    this.redoIndex = -1;
    
    var undoElem = null;
    this.undoElem = undoElem;
    
    var undoTR =  null;
    var undoProps = null;
    var undoTextItems = null;
    this.undoTR = undoTR;
    this.undoProps = undoProps;
    this.undoTextItems = undoTextItems;
};

//This is the function that creates all the commands.
//These are the same types of commands that
//appear in FrameMaker menus, except we aren't
//adding them to any menus. We need a command, though,
//in order to create a shortcut.
function DefineCommands()
{
    var cmd;
    
    //Step through the list of commands in the global
    //array and create them, giving the first command
    //a number of zero and incrementing from there.
    //By numbering commands like this, we know that when
    //a command is executed, the integer that is sent to the
    //Command() callback will align with the array of
    //commands in the globals; that is, the command number
    //will act as an index that allows us to locate the desired
    //formatting parameters in the globals that correspond 
    //to the command.
    for(var i = 0; i < sGlobals.shortcuts.length; i++)
    {
        if(sGlobals.shortcuts[i] != "")
            cmd = DefineCommand(i, 
                "Format_Shortcut_Script_Cmd_" + i, 
                "Format Shortcut Script Cmd " + i,
                sGlobals.shortcuts[i]);
    }

    //If an undo shorcut was defined, let's create a command for it.
    if(sGlobals.undoShortcut != "")
        cmd = DefineCommand(i, 
            "Format_Shortcut_Script_Cmd_Undo", 
            "Format Shortcut Script Cmd (Undo)",
            sGlobals.undoShortcut);

    //If a redo shortcut was defined, let's create a command for it.
    if(sGlobals.redoShortcut != "")
        cmd = DefineCommand(i + 1, 
            "Format_Shortcut_Script_Cmd_Redo", 
            "Format Shortcut Script Cmd (Redo)",
            sGlobals.redoShortcut);

    //If a command catalog shortcut was defined, let's create a command for it.
    if(sGlobals.catalogShortcut != "")
        cmd = DefineCommand(i + 2, 
            "Format_Shortcut_Script_Cmd_Catalog", 
            "Format Shortcut Script Cmd (Catalog)",
            sGlobals.catalogShortcut);

    //If an unwrap shortcut was defined, let's create a command for it.
    if(sGlobals.unwrapShortcut != "")
        cmd = DefineCommand(i + 3, 
            "Format_Shortcut_Script_Cmd_Unwrap", 
            "Format Shortcut Script Cmd (Unwrap)",
            sGlobals.unwrapShortcut);

};

//This is the master function that performs the
//formatting action. It is sent an index which
//is used to look up the correct formatting parameters in the
//globals. This index is the same as the command number
//that was executed... we numbered the commands like this to
//make it simple.
function ApplyFormatting(index)
{
    //if something is bogus, abort.
    if(index < 0 || index > sGlobals.shortcuts.length) return;
    
    //Get the active document, without which there is nothing to do.
    var doc = app.ActiveDoc;
    if(!doc.ObjectValid())
    { 
        alert(sGlobals.msgHeader + "No active document. Cannot apply formatting");
        return;
    }

    //Let's remember the formatting parameters that
    //we are about to apply, by means of remembering the
    //index. That way, if we want to redo the formatting
    //(with the redo command) on another range of text,
    //we can just call this function with that index.
    sGlobals.redoIndex = index;

    //If the global settings at the index contain an element tag,
    //let's assume that this is a structure-based format action
    //and we want to wrap an element.
    if(sGlobals.elemTags[index] != "")
    {
        //Get the element definition from the EDD, required for the wrap.
        //If we can't get it, warn and abort.
        var elemDef = doc.GetNamedObject (Constants.FO_ElementDef, sGlobals.elemTags[index]);
        if(!elemDef.ObjectValid())
        {
            alert(sGlobals.msgHeader +
                "The specified element to wrap for formatting, <" + 
                sGlobals.elemTags[index] + ">, does not appear " +
                "to exist in the current EDD. Check your script settings.");
                
            return;
        }
    
        //...and one simple call to wrap the currently-selected
        //text with the specified element. We don't have to
        //set up any kind of element range, because the user
        //should have wanted to format the currently-selected text,
        //so everything is already set up.
        elemDef.WrapElement();
        
        //Now let's get the object of the new element that was
        //inserted. WrapElement() unfortunately does not
        //return the new element object, so we have to use
        //a little trickery based on where the insertion point
        //is to figure it out. This is a bit complicated to understand.
        var newElem = doc.ElementSelection.beg.child;
        if(newElem.id ==0)
            newElem = doc.ElementSelection.beg.parent;
       
        //So, if we got the new element object....        
        if(newElem.id != 0)
        {
            //Store the object in the undo global
            //so we can unwrap it later if the redo command
            //is executed.
            sGlobals.undoElem = newElem;
            sGlobals.undoTR = null;
            
            //If the global settings at the current index
            //indicate an attribute name, let's set that attribute
            //with the specified value.
            if(sGlobals.attrNames[index] != "")
            {
                //SetAttribute() is a custom function in this script,
                //defined later.
                SetAttribute(newElem, sGlobals.attrNames[index], -1, 
                    sGlobals.attrVals[index]);
            }
        
            //If the new element has no contents, let's assume we did
            //an insertion and put the insertion point inside the new element.
            var childElem = newElem.FirstChildElement;
            var tr = newElem.TextRange;
            var ti = doc.GetTextForRange(tr, Constants.FTI_String);

            if(!childElem.ObjectValid() && ti.len < 1)
            {
                er = new ElementRange();
                er.beg.parent = er.end.parent = newElem;
                //er.beg.child = er.end.child = elem.FirstChildElement;
                er.beg.offset = er.end.offset = 0; 
                doc.ElementSelection = er;
            }
        }
    }
    
    //If there is no element tag specified but we do have
    //a character format, this is an unstructured format action,
    //that is, the application of a character format.
    else if(sGlobals.chrFormats[index] != "")
    {
        //Get the character format object from the template.
        //If we have any trouble, warn and abort.
        var chrFmt = doc.GetNamedObject (Constants.FO_CharFmt,
            sGlobals.chrFormats[index]);
        if(!chrFmt.ObjectValid())
        {
            alert(sGlobals.msgHeader +
                "The specified character format to apply for formatting, \"" + 
                sGlobals.chrFormats[index] + "\", does not appear " +
                "to exist in the current template. Check your script settings.");
                
            return;
        }

        //Before we apply the format, let's capture an undo
        //snapshot  which will be used
        //to undo the action if the undo command is executed.
        //We use a custom function for this... it is a bit
        //involved and complex, much different than 
        //a structural undo which is a simple element unwrap.
        CaptureChrFormatUndoSnapshot(doc);
        sGlobals.undoElem = null;

        //Now apply the format. Pretty simple... the way you
        //do it is to retrieve the properties from the format
        //and apply them to a range of text, doc.TextSelection
        //in this case (that is, the text currently selected by the user).
        var props = chrFmt.GetProps();
        doc.SetTextProps(doc.TextSelection, props);
    }

    //If we didn't have an element tag or a character format, warn that the
    //global settings are jacked up.
    else
    { 
        alert(sGlobals.msgHeader + "The formatter script attempted an action," +
            "but the defined shortcut has no element tag or " +
            "character format configured. Check the script settings.");
    }
} ;  
 

//This is a function to capture a snapshot of the selected
//text range before applying a format. It is somewhat 
//complicated... we have to find every instance 
//of unique character formatting, then store the location
//and details of that formatting so that it can be restored
//upon an undo action.
//You have to be well versed in scripting for FM to 
//make any sense of this. Comments are kept to a minimum.
function CaptureChrFormatUndoSnapshot(doc)
{
    //We'll use this variable to track along the
    //locations of format changes.
    var textLoc = doc.TextSelection.beg;
    
    //Store the overall text range where formatting
    //will be applied.
    sGlobals.undoTR = doc.TextSelection;
    
    //Start building the array of property lists
    sGlobals.undoProps = new Array();
    
    //Call GetTextForRange() to discover all the
    //offsets where a character format change occurs.
    //We'll keep this in the globals because we'll need
    //those offsets later during a restoration.
    sGlobals.undoTextItems = 
        doc.GetTextForRange (doc.TextSelection, 
            Constants.FTI_CharPropsChange);
    
    //Store the format properties at the beginning of the range.
    //We always do that... always at least one property list.
    sGlobals.undoProps.push(doc.GetTextProps(textLoc));
            
    //Now, for each location where a format change occurred,
    //store the property list in the global array.
    for(var i = 0; i < sGlobals.undoTextItems.length; i++)
    {
        textLoc.offset = sGlobals.undoTextItems[i].offset;
        sGlobals.undoProps.push(doc.GetTextProps(textLoc));
    }    
}

//This function uses the stored global snapshot to restore
//formatting on an undo action. Again, this is complicated.
//Comments are kept to a minimum.
function UndoChrFormatApply()
{
    if(sGlobals.undoTR == null) return;
    
    var doc = app.ActiveDoc;
    if(!doc.ObjectValid()) return;
    
    //Get a new text range based on the original
    //restoration range stored earlier.
    var tempTR = new TextRange();
    tempTR.beg.obj = sGlobals.undoTR.beg.obj;
    tempTR.beg.offset = sGlobals.undoTR.beg.offset;
    tempTR.end.obj = sGlobals.undoTR.end.obj;
    tempTR.end.offset = sGlobals.undoTR.end.offset;
    
    //If there was at least one format change in the range
    //originally, we'll have at least one item in the global
    //text items list. In this case, we can shorten the text
    //range to just the first part of the string before any change
    //occurred.
    if(sGlobals.undoTextItems.length > 0)
        tempTR.end.offset = sGlobals.undoTextItems[0].offset;
        
    //Set the first set of text props on the current range.    
    doc.SetTextProps(tempTR, sGlobals.undoProps[0]);

    //Now, if there were any format changes along the range,
    //we will have some text items in the global array and 
    //corresponding property lists in that other array for each
    //text item offset. Step through each one, finangle the offsets
    //and apply the properties.
    for(var i = 0; i < sGlobals.undoTextItems.length; i++)
    {
        //The beginning offset is always the current text item offset;
        //that is, the place where the format change occurred.
        tempTR.beg.offset = sGlobals.undoTextItems[i].offset;

        //The end offset is either the next offset where a change occurred,
        //or, if we are at the end of the list of changes, the end of the original
        //range
        if((i + 1) < sGlobals.undoTextItems.length)
            tempTR.end.offset = sGlobals.undoTextItems[i + 1].offset;
        else tempTR.end.offset = sGlobals.undoTR.end.offset;
        
        //and apply.
        doc.SetTextProps(tempTR, sGlobals.undoProps[i + 1]);
    }
}

//Here is the callback function that is called
//automatically whenever one of our commands is
//executed. This is a standard callback that any script
//must have if it defines commands that require action 
//when executed. The cmd argument is the
//the number of the command, assigned when
//the command was created with our
//DefineCommands function
function ExecuteCommand(cmd)
{
    //alert(cmd);
    
    //If the command number is within the list
    //of format settings in the globals, this is a format
    //command. Send the command number to the
    //formatter function which will retrieve the proper
    //format settings based on that index.
    if(cmd > -1 && cmd < sGlobals.shortcuts.length)
    {
        ApplyFormatting(cmd);
    }
    
    //If the command number equals the number of
    //setting groups in the globals, it is the undo command.
    //This is just the way we numbered commands when
    //they were defined.
    else if(cmd == sGlobals.shortcuts.length)
    {
        //We'll need a valid document object
        var doc = app.ActiveDoc;
        
        if(doc.ObjectValid())
        {
            //Let's store the current insertion point/text
            //selection, so we can restore it after the undo
            //action, in the event that the undo action
            //changes the current insertion point location.
            var restoreTR = doc.TextSelection;
            
            //If we have an "undo element tag",
            //the last format action was a structural wrap.
            //Let's just unwrap that element.
            if(sGlobals.undoElem != null)
            {
                //This is a custom function defined later in the
                //script.
                UnwrapElement(doc, sGlobals.undoElem);
                sGlobals.undoElem = null;
            }
            
            //If instead we stored an "undo text range",
            //the last format action was a character
            //format application. Let's call this custom
            //function to handle it.
            else if(sGlobals.undoTR != null)
            {
                UndoChrFormatApply();
                sGlobals.undoTR = null;
            }
            
            //Restore the original insertin point/text selection
            //in case the undo action moved it.
            doc.TextSelection = restoreTR;
        }
    }
    
    //If the command number is one greater than the number
    //of settings groups, it is the redo command. We can
    //just call the formatter function with the index that was
    //used originally (that is, the original command number).
    //We remembered it in the globals when the formatting was
    //first applied.
    else if(cmd == sGlobals.shortcuts.length + 1)
    {
        ApplyFormatting(sGlobals.redoIndex);
    }

    //If the command number is two greater than the number
    //of settings groups, it is the command to launch the catalog
    else if(cmd == sGlobals.shortcuts.length + 2)
    {
        LaunchCommandCatalog();
    }

    //If the command number is three greater than the number
    //of settings groups, it is the command to unwrap the current element
    else if(cmd == sGlobals.shortcuts.length + 3)
    {
        var doc = app.ActiveDoc;
        
        if(doc.ObjectValid())
        {
            var elem = null;
            var er = doc.ElementSelection;
        
            //Try to figure out what element is selected or contains
            //the insertion point.
            if(er.beg.child.ObjectValid() &&
                (er.beg.child.id != er.end.child.id)) elem = er.beg.child;
        
            else if(er.beg.parent.ObjectValid() && (er.beg.parent.id == er.end.parent.id))
                elem = er.beg.parent;
                
            if(elem != null)
            {
                er.beg.parent = er.end.parent = elem.ParentElement; 
                er.beg.child = elem;
                er.end.child = elem.NextSiblingElement;
                er.beg.offset = er.end.offset = 0;

                doc.ElementSelection = er;
                doc.UnWrapElement();
            }
            
            else alert("No element selected. Cannot unwrap.");
        }
        else alert("No active document. Cannot unwrap.");
    } //end main else if

};


//A convenience function for setting an element
//attribute value. Send an attribute name or index, not
//both. It overwrites any current values. It returns true if it
//thinks the action was successful.
//
// elem = The element object
// attrName = The attribute name. If using an index instead,
//                       send an empty string
// attrIndex = The attribute index, as it would appear in an
//                      Attributes array. If using a name instead,
//                       send a -1.
// newValue = The attribute value to set.
function SetAttribute(elem, attrName, attrIndex, newValue)
{
    if(!elem.ObjectValid()) return false;
    
    var returnVal = false;
    
    var attrs = elem.Attributes;

    for(var i = 0 ; i < attrs.length; i++) 
    {
        if (attrs[i].name == attrName ||
            i == attrIndex) 
        {
            attrs[i].values = new Strings();
            attrs[i].values.push(newValue);
            
            elem.Attributes = attrs;
            i = attrs.length;
            returnVal = true;
        }
    }

    return returnVal;
};

//A convenience function for unwrapping an element.
//You have to select it in the document before you
//can unwrap it, which is what most of this code does.
//Afterwards, the unwrap is a single call.
function UnwrapElement(doc, elem)
{
    if(elem == null || !doc.ObjectValid() || !elem.ObjectValid()) return;
    
    var er = new ElementRange;
                
    er.beg.parent = er.end.parent = elem.ParentElement; 
    er.beg.child = elem;
    er.end.child = elem.NextSiblingElement;
    er.beg.offset = er.end.offset = 0;
    
    doc.ElementSelection = er;
    
    doc.UnWrapElement();
};



function LaunchCommandCatalog()
{
	var res =
		"""palette {
			properties:{ closeButton:true, maximizeButton:false,
				minimizeButton:false, resizeable:true },
			orientation:'column', spacing:2, margins:2,
              cmdsLb: ListBox {
                  alignment:['fill','fill'],
                  properties:{numberOfColumns: 2, showHeaders:true, 
                  columnTitles: ['Description', 'Shortcut'], columnWidths:[100,1000]}
              },
             applyBtn: Button { 
                  text:'Apply', alignment:['fill','bottom'] },
        }""";   
        
    var win = new Window (res, "Format Commands");
    
    var lbItem;
    //lbItem = win.cmdsLb.add("item", "Temp string 1234567890");
    //lbItem.subItems[0].text = "Temp string 123456";
    win.layout.layout(true);
    //win.cmdsLb.remove(0);

    for(var i = 0; i < sGlobals.shortcuts.length; i++)
    {
        lbItem = win.cmdsLb.add("item", sGlobals.descriptions[i]);
        lbItem.subItems[0].text = 
            MakeReadableShortcut(sGlobals.shortcuts[i]);
    }                

    lbItem = win.cmdsLb.add("item", "Undo");
    lbItem.subItems[0].text = 
        MakeReadableShortcut(sGlobals.undoShortcut);
    lbItem = win.cmdsLb.add("item", "Redo");
    lbItem.subItems[0].text = 
        MakeReadableShortcut(sGlobals.redoShortcut);
    lbItem = win.cmdsLb.add("item", "Unwrap");
    lbItem.subItems[0].text = 
        MakeReadableShortcut(sGlobals.unwrapShortcut);

    win.layout.layout(true);
    win.show();
    win.bounds = {x:100, y:100, width:175, height:200};
    win.layout.resize();

    win.cmdsLb.onDoubleClick = function() 
    {
        if(win.cmdsLb.selection != null)
            ExecuteCommand(win.cmdsLb.selection.index);
	}
    win.applyBtn.onClick = function() 
    {
        if(win.cmdsLb.selection != null)
            ExecuteCommand(win.cmdsLb.selection.index);
	}

    win.onResize = function() 
    {
        win.layout.resize();
        
	}
 }

function MakeReadableShortcut(shortcut)
{
    var returnStr = "";
    var chrs = shortcut.split("");
    
    for(var i = 0; i < chrs.length; i++)
    {
        if(chrs[i] == "^") returnStr += "Ctrl+";
        else if(chrs[i] == "+") returnStr += "Shift+";
        else if(chrs[i] == "\\" &&
            (i + 1) < chrs.length && 
            chrs[i+1] == "!")
        {
            returnStr += "Alt ";
            i++;
        }
        else returnStr += chrs[i].toUpperCase() + " ";
        
    }

    return returnStr;
    
}    

