﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (none)

DESCRIPTION:
    
This script creates a menu named "Favorites"
and adds a list of links to files, as menu "commands."
The file links are
customizable, as well as where to place the
Favorites menu. READ THESE INSTRUCTIONS
CAREFULLY so that you know how to make
the script create a menu that works for you.

Note: You can run the script as-is. Afterwards, you should
see a new Favorites menu in the main 
menu bar and in the File menu. The 
favorites will be bogus, but you can 
at least see the general idea of how it works.

To make it actually work for you,
there are two things you need to configure:

  1 - The place(s) that the menu should go.
         Just below, this comment block will end
         and some additional line comments will
         explain more. Then, continue reading with the
         second comment block afterwards.
*/   


    //Here is where you should 
    //define the parent menu(s) for the Favorites
    //menu. For each parent menu, add a new push()
    //line with the name of the menu.
    //
    //These menu names are something that FM uses
    //internally. They are not necessarily
    //intuitive and it takes experience to know them
    //or know how to find them out. If there is a menu
    //name that you need to know, please contact
    //West Street.
    //
    //Some common names are:
    //   !MakerMainMenu - The main document menu
    //   !BookMainMenu - The main book menu
    //   FileMenu - The File menu when a document is active
    //    BookFileMenu - The File menu when a book is active
    //
    //If you wanted to get fancy, you could add separators, reorder
    //menu items, etc. This script doesn't do any of that. It simply
    //creates adds the menu at some default location. probably
    //at or close to the end.,

    var parentMenus = new Array(0);
    parentMenus.push("!MakerMainMenu");
    parentMenus.push("!BookMainMenu");
    parentMenus.push("FileMenu");
    parentMenus.push("BookFileMenu");




/*
    
    Here begins the second comment block,
    describing the second thing you need to
    //configure... the actual file links that will
    //go into the menu; that is, your favorites.
    //You have two options:
    
       1 - Define them in this script. If that is
            what you want to do, set the  
             favoritesFilePath variable (below)
            to an empty path, then go to the 
            defineFavorites() function for further 
            instructions.
            
        -or-
            
       2 - Specify them in a text file. If you 
            choose this option, set the 
            favoritesFilePath to the path of that
            file, then create the text file with entries 
            in this format:

<path>~~~~<menu_text>

          ....where <path> is the path to the file
          and <menu_text> is the text
          you want to appear in the Favorites menu.
          The delimiter of 4 tildes is very important.
          
          If you want the menu text to be the same 
          as the path, just omit the text, for example:
          
<path>

        If you want to add a menu separator, specify
        "separator", for example:
        
separator

         If you want to comment out a line, precede
         it with a semicolon.
         
         Here is a sample, hypothetical file.

C:\docs\MyFile.fm~~~~My File
C:\docs\SomeOtherFile
separator
;Some line that is just commented out.
C:\docs\SomeBook.book~~~~My FrameMaker Book
C:\docs\SomeOtherBook.book
           
            
            Got all that? Once all that is set
            up, then run the script. More than likely,
            once you get it working right, you will
            want it to run automatically at FM startup.
            You can do that in the FM interface in the
            script catalog, File > Script > Catalog. See
            the ES documentation for more information.
            
            Keep reading below to make sure the
            favoritesFilePath variable is set correctly.
*/


//VERY IMPORTANT SETTING HERE. If you are specifying
//favorites in an external text file, specify that path here. Use
//double backslashes to indicate a file delimiter. If you are 
//instead specifying the favorites directly in this script,
//specify an empty string and go to defineFavorites() below
//for more instruction.
//
// Here are examples:
//var favoritesFilePath = "C:\\somefolder\\MyFavorites.txt";
//var favoritesFilePath = "";

var favoritesFilePath = "";
//var favoritesFilePath = "C:\\temp\\delete\\favorites.txt";


//This needs to be global, so  we'lldeclare it here.
var favorites;

//Another global which represents the internal
//name that will be assigned to your new menu.
//It should not need to be changed unless it 
//happens to conflict with an existing FM menu.
var favoritesMenuName = "MyFavoritesMenu";

//Here is where the main action is launched.
createFavoritesList(favoritesFilePath);

//This is the main function that drives the activity.
function createFavoritesList(favoritesFilePath)
{
    //This script really should be run only once per session.
    //Let's see if the favorites menu already exists
    //and if so, assume that the script has already been run.
    var menu = app.GetNamedMenu(favoritesMenuName);
    if(menu.ObjectValid())
    {
        if(!confirm("It appears that you have already "
            + "run this script once during this session. "
            + "Manipulation of existing menu commands "
            + "usually does not work very well in FrameMaker. "
            + "An FM restart is recommended.\n\nDo you want "
            + "to continue anyway?"))
                return;
    }
            
    //Define the favorites for the menu.
    favorites = defineFavorites(favoritesFilePath);
    
    //If something went wrong, abort.
    if(favorites.length < 1)
    {
        alert("No valid favorites are defined. "
            + "The favorites list cannot be created.");
        
        return;
    }
    
    //Otherwise, create the menu and stick it
    //in the defined parentMenus.
    createAndInstallFavoritesMenu(parentMenus);
    
    alert("Script complete. Favorites menu created "
        + "(or at least attempted).");

}

//This function defines the favorites for the menu.
//It just defines them. It does not add them to the
//menu.
//IF YOU CHOSE TO SPECIFY FAVORITES
//DIRECTLY IN THE SCRIPT, this is where you do
//it. Read on.
function defineFavorites(favoritesFilePath)
{
    var favorites = new Array(0);
    
    //If no favorites file path is specified,
    //the favorites should be configured here.
    if(favoritesFilePath == "")
    {
        var favorite;
        
        //For each favorite you want, add
        //a block here like this (without
        //the comment slashes, of course):
        
        //favorite = new Array(2);
        //favorite[0] = "C:\\docs\\SomeFile.fm";
        //favorite[1] = "Some File";
        //favorites.push(favorite);
        
        //The first member of the array is the path.
        //Use double-backslashes for path delimiters.
        //The second member is the text to appear
        //in the menu. If you want the text to be the
        //same as the path, specify an empty string.
        
        //To add a menu separator, specify 
        //"separator" for the first member.
        
        //After here are some examples,
        //Naturally, they probably represent bogus files
        //but it should show you how to do it.
        //You can run the script with these if you want...
        //the menu commands just won't open anything.
        
        favorite = new Array(2);
        favorite[0] = "C:\\docs\\SomeFile.fm";
        favorite[1] = "Some File";
        favorites.push(favorite);
        
        favorite = new Array(2);
        favorite[0] = "C:\\docs\\SomeOtherFile.fm";
        favorite[1] = "";
        favorites.push(favorite);
        
        favorite = new Array(2);
        favorite[0] = "separator";
        favorite[1] = "";
        favorites.push(favorite);
        
        favorite = new Array(2);
        favorite[0] = "C:\\docs\\SomeBook.fm";
        favorite[1] = "My favorite book";
        favorites.push(favorite);
    }

    //If instead you are reading favorites from a file,
    //this is the routine that does it. You don't need
    //to do anything here. You just need to have the
    //file correctly formatted and the right path
    //sent here.
    else
    {
        //See if the file exists
        var file = new File(favoritesFilePath);
        if(file.exists)
        {
            //If it exists, open it for reading
            file.open("r");
            
            //Read line by line until the end is reached.
            while(!file.eof)
            {
                 //Get the next line
                 var line = trim(file.readln());

                 //As long as the line isn't empty or starts
                 //with a semicolon (that is, it's commented out)
                 //let's process it as a favorite.
                 if(line != "" && line.indexOf(";") != 0)
                 {
                     //Split the string at the 4-tilde delimiter
                     var args = line.split("~~~~");
                     
                     //Initialize the favorite array to add to the
                     //main favorites list
                     var favorite = new Array(2);
                     
                     //The first member is the path.
                     favorite[0] = args[0];
                     
                     //If we got a second member when
                     //we split the string, that will be the
                     //menu text. Otherwise, it was not
                     //specified and we should add
                     //an empty string.
                     if(args.length > 1)
                         favorite[1] = args[1];
                     else favorite[1] = "";
 
                     favorites.push(favorite);
                 }
             }
        }
    
        //If we could not find the favorites file at the specified
        //path, let's warn.
        else
        {
            alert("The favorites definition file at:\n\n" 
                + favoritesFilePath
                + "\n\n...could not be found. The favorites "
                + "list cannot be created.");
        }
    
        //Close the I/O channel to the file
        file.close();
    }
    
    //Return the master list.
    return favorites;
}

//This function creates and installs the Favorites menu.
//The main favorites list must be created first.
function createAndInstallFavoritesMenu(parentMenus)
{
    //Let's create the favorites menu object.  This creates
    //the menu but doesn't place it anywhere.
    var favoritesMenu = 
        DefineMenu(favoritesMenuName, "Favorites");

    //Now, iterate over the favorites list
    //and create a command for each, assigning
    //a number to it that corresponds with 
    //the index in the list. Later, when the
    //command is called, that is how we will
    //identify the correct path.
    
    //This may be a bit complicated if you
    //don't understand the process of command
    //creation and response yet. See the 
    //basic sample script for menu and command
    //creation to for more extensive information.
    for(var i = 0; i < favorites.length; i++)
    {
        //Create a separator if that's what was specified.
        if(favorites[i][0].toLowerCase() == "separator")
        {
            var cmd = 
                app.NewNamedObject(Constants.FO_MenuItemSeparator, 
                    "Favorites_Separator_" + i);
        }
    
        //Otherwise, create a command. 
        else 
        {
            //The menu needs some text. If it was specified,
            //use it. Otherwise, use the path.
            var menuText;
            if(favorites[i][1] == "")
                menuText = favorites[i][0];
            else menuText = favorites[i][1];
            
            var cmd = 
                DefineCommand(i, "MyFavorite" + i, menuText, "");
                
        }
        
        //Add the command to the Favorites menu
        favoritesMenu.AddCommandToMenu(cmd);
    }

    //Now, put the Favorites menu in the specified parent menus
    //so it will actually appear somewhere.
    for(var i = 0; i < parentMenus.length; i++)
    {
        var fmMenu = app.GetNamedMenu(parentMenus[i]);
        if(fmMenu.ObjectValid())
        {
            fmMenu.AddMenuToMenu(favoritesMenu);
        }
    
        else
        {
            alert("The favorites list generator attempted to find an FM "
                + "menu named \"" + parentMenus[i]
                + "\" but the menu could not be found. Therefore, the "
                + "Favorites submenu cannot be added to it. "
                + "Check the syntax of the menu name.");
        }
    }

    //This is a built-in function that is required to refresh 
    //everything and actually make the new menu appear
    UpdateMenus();
}

//This is the function required for a script
//to respond to commands it created.
//The cmd argument will be populated
//with the number of the command, assigned
//when the command was created. In this script,
//that will correspond to the index of the
//respective favorite in the main favorites list.
function Command(cmd)
{
    //Look in the favorites list for the
    //path.
    var path = favorites[cmd][0];
    
    //Use a File object to see if the file
    //even exists. If it does not, we'll give
    //the user the option to back out.
    var file = new File(path);
    if(!file.exists)
    {
        if(!confirm("The file: \n\n" 
            + path
            + "\n\n...does not appear to exist. Do "
            + "you want to attempt to open it anyway?"))
                return;
    }

    file.close();

    //Try to open the file. This function is defined
    //later in this script and provides some control
    //over the file opening process.
    var file = OpenFile(path, true, false, false, true);
    
    //If it seems it did not open, warn.
    if(!file.ObjectValid())
    {
        alert("The file: \n\n" 
            + path
            + "\n\n...could not be opened. "
            + "It may be in-use, non-existent, or corrupt. "
            + "Try opening the file manually to determine the trouble");
    }
}

//Here is a convenience function for opening files. Lots of stuff
//going on here and there are no comments within. Study up on ES
//documentation to understand more.
function OpenFile(path, ignoreErrors, ignoreLock, hidden, allowNonWriteable)
{
    var index;

    props = GetOpenDefaultParams();

    index = GetPropIndex(props, Constants.FS_AlertUserAboutFailure);
    if(ignoreErrors)
        props[index].propVal.ival = false;
    else
        props[index].propVal.ival = true;
        
    index = GetPropIndex(props, Constants.FS_BookIsInUse);
    if(ignoreLock)
        props[index].propVal.ival = Constants.FV_ResetLockAndContinue;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_DisallowMIF);
    props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_DisallowXml);
    props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_DontNotifyAPIClients);
    props[index].propVal.ival = true;
        
    index = GetPropIndex(props, Constants.FS_FileIsOldVersion);
    props[index].propVal.ival = Constants.FV_DoOK;
        
    index = GetPropIndex(props, Constants.FS_FileIsInUse);
    if(ignoreLock)
        props[index].propVal.ival = Constants.FV_ResetLockAndContinue;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_FontChangedMetric);
    if(ignoreErrors)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_FontNotFoundInCatalog);
    if(ignoreErrors)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_FontNotFoundInDoc);
    if(ignoreErrors)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_LanguageNotAvailable);
    if(ignoreErrors)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_LockCantBeReset);
    if(ignoreLock)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_MakeVisible);
    if(hidden)
        props[index].propVal.ival = false;
    else
        props[index].propVal.ival = true;
        
    index = GetPropIndex(props, Constants.FS_NewDoc);
    props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_OpenAsType);
    props[index].propVal.ival = Constants.FV_AUTORECOGNIZE;
        
    index = GetPropIndex(props, Constants.FS_OpenBookViewOnly);
    props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_OpenDocViewOnly);
    props[index].propVal.ival = false;
        
    index = GetPropIndex(props, Constants.FS_UseAutoSaveFile);
    props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_UpdateTextReferences);
    props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_UpdateXRefs);
    props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_UseRecoverFile);
    props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_UseAutoSaveFile);
    props[index].propVal.ival = Constants.FV_DoNo;
        
    index = GetPropIndex(props, Constants.FS_OpenFileNotWritable);
    if(allowNonWriteable)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_DoCancel;
        
    index = GetPropIndex(props, Constants.FS_RefFileNotFound);
    if(ignoreErrors)
        props[index].propVal.ival = Constants.FV_DoOK;
    else
        props[index].propVal.ival = Constants.FV_AllowAllRefFilesUnFindable;
  
    //open the document
    returnp = new PropVals();
    var file = Open(path, props, returnp);
  
    if(file.ObjectValid() && hidden)
        file.IsOnScreen = false;
        
    return file;
}    

// A shortcut function to strip leading and trailing
// whitespace from a string
function trim(str)
{
    while(str.indexOf(" ") == 0 && str.length > 0)
        str = str.substring(1, str.length);
    while(str.lastIndexOf(" ") == (str.length - 1) && str.length > 0)
        str = str.substring(0, str.length - 1);
        
    return str;
}    
