﻿/*  ....Commentary....

REQUIRED ACTIVE FILE:  (none)

DESCRIPTION:
    
This script adds up to three keyboard shortcuts
to set the display of attributes in the Structure View,
either:
   
     - All attributes displayed (default shortcut: Esc a 1)
     - No attributes displayed (default shortcut: Esc a 2)
     - Only populated and required attributes displayed 
       (default shortcut: Esc a 3)

All you have to do is configure the shortcuts you want, then
run the script. The shortcuts will be ready for use. Note that:

     - All operations act upon all elements in the main flow.
       The current selection is not considered, but could be
       if you wanted to enhance the script a bit.
       
     - For any of the three operations, if you don't want a
       command for it, specify an empty string for the shortcut.
       
     - The commands created here are like any other command
        and could be added to menus. This script doesn't go that
        far, but if you are clever enough, you could add that part.

    Here are some tips on how to format a shortcut:
    
     A shortcut is represented as a string that begins with at 
     least one special character, then a "normal" character.
     Here are the special characters that work well on Windows:
     
     \\!     - Esc key
     ^      - Ctrl key
     +      - Shift key
     
     Here are some examples:
     
     ^b                - Ctrl + B (press and hold keys in sequence)
     ^+b              - Ctrl + Shift + B  (press and hold keys in sequence)
     \\!b               - Esc + B (Esc first, then B)
     \\!bb            - Esc + B + B (Esc first, then B, then B again)
     
     IMPORTANT! Any shortcut you specify here will override 
     any native, default shortcut with the same sequence! For 
     example, if you define Ctrl + B, then the normal functionality
     of Ctrl + B will be overridden and only the code in this
     script will be executed.
*/ 

//HERE ARE WHERE THE SHORTCUTS ARE DEFINED.
//If you don't want a command for any of these, just
//specify an empty string. Once these are configured,
//the script is ready to be run.
var collapseAttrsShortcut = "\\!a1";
var expandAttrsShortcut = "\\!a2";
var showPopAndReqAttrsShortcut = "\\!a3";


//Here is where the main action is launched.
var commandsCreated = createCommands();

alert("Script complete - " + commandsCreated
     + " command(s) created. If this number does "
     + "not seem right, check the shortcut definitions "
     + "in the script.");

//This function creates the commands
function createCommands()
{
    var commandsCreated = 0;
    
    //Create each of the three commands, as applicable.
    //They will be numberd with a simple 1, 2, and 3
    if(collapseAttrsShortcut != "")
    {
          var cmd = 
                DefineCommand(1, "CollapseAllAttrDisplays", 
                    "Collapse Attribute Displays", collapseAttrsShortcut);
                    
          if(cmd.ObjectValid()) commandsCreated++;
    }

    if(expandAttrsShortcut != "")
    {
          var cmd = 
                DefineCommand(2, "ExpandAllAttrDisplays", 
                    "Expand Attribute Displays", expandAttrsShortcut);
                    
          if(cmd.ObjectValid()) commandsCreated++;
    }

    if(showPopAndReqAttrsShortcut != "")
    {
          var cmd = 
                DefineCommand(3, "ShoePopAndReqAttrDisplays", 
                    "Show Pop. And Req. Attrs", showPopAndReqAttrsShortcut);
                    
          if(cmd.ObjectValid()) commandsCreated++;
    }

    //Send back the number of commands created.
    return commandsCreated;
}

//This is the function that does the work of configuring
//attribute displays. Send it the setting you want, one of:
//
// - Constants.FV_ATTR_DISP_NONE
// - Constants.FV_ATTR_DISP_ALL
// - Constants.FV_ATTR_DISP_REQSPEC
function configureAttrDisplays(displaySetting)
{
    //Get the active document
    var doc = app.ActiveDoc;
    if(!doc.ObjectValid())
    {
        alert("No active document. Cannot continue.");
        return;
    }

    //We will keep a counter to prevent an endless
    //loop while iterating over elements.
    //This should never need to be used; however,
    //experience has shown that occasionally
    //a structure tree will get corrupted in some way
    //that the element navigator gets stuck.
    var endlessLoopCounter = 0;
    
    //Get the highest level element of the main flow.    
    var hle = elem = doc.MainFlowInDoc.HighestLevelElement;
    
    //Iterate over all elements in the tree and set the display setting
    while(elem.ObjectValid() && endlessLoopCounter < 100000)
    {
        if(elem.ElementDef.Name != "")
            elem.AttrDisplay = displaySetting;
        elem = elem.NextElementDFS;
    }

    //If the highest-level element is currently expanded
    //in the Structure View, let's toggle it collapsed and back.
    //This should clean up any display weirdness left behind
    //from our operation.
    if(hle.ObjectValid() && !hle.ElementIsCollapsed)
    {
        hle.ElementIsCollapsed = true;
        hle.ElementIsCollapsed = false;
    }
}

//Here is the standard function to respond to commands
//created by the script. We'll just look for one
//of the three we created and then lauch the display
//setter with the appropriate setting.
function Command(cmd)
{
    if(cmd == 1)
        configureAttrDisplays(Constants.FV_ATTR_DISP_NONE);
    
    else if(cmd == 2)
        configureAttrDisplays(Constants.FV_ATTR_DISP_ALL);
    
    else if(cmd == 3)
        configureAttrDisplays(Constants.FV_ATTR_DISP_REQSPEC);
}


